unit An8255CardFrame;

interface

uses 
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, An8255Frame, StdCtrls;

type
  TOne8255CardFrame = class(TFrame)
    ACardPanel: TPanel;
    CardInfoPanel: TPanel;
    Cards8255Panel: TPanel;
    ACardDescriptionLabel: TLabel;
  private
    { Private declarations }
  public
    DeviceID: LongWord;
    Num8255: LongWord;
    Name: String;
    Note: String;
    Group: Array of TOne8255Frame;
    constructor Create(AOwner: TComponent; destPanel:TPanel; FrameName: string; C, G, X, Y:integer); overload;
    function Show8255Frame(C, G, X, Y:integer): TOne8255Frame;
    function BuildCardFrames( C, X, Y:integer):Boolean;
    { Public declarations }
  end;

implementation

uses pcidio8255Unit1, aiowdm;

{$R *.DFM}

{ TOne8255CardFrame }

constructor TOne8255CardFrame.Create(AOwner: TComponent; destPanel:TPanel; FrameName: string; C, G, X, Y:integer);
  const
    bmDIO8255 = $0EFF;                                // Pure DIO boards can only have these bits set
    bmDIOType = $00C0;                                // These bits set what type of DIO is on the card
    bDIOTypeIs8255 = $0040;                           // the DIOType bits must be this value to be 8255-style DIO
    bmNumChannels = $0031;                            // These 3 bits enumerate how many DIO bits are on the card
    bmIsolated = $0100;                               // This bit is set on Isolated devices
    NumChannelsToNumGroups: array[0..7] of integer = (0, 1, 1, 2, 2, 3, 4, 5);
  var
    N: Integer;
begin
  try
    inherited Create(nil);
  except
    showmessage('bug');
  end;
  begin
    self.Name := FrameName + IntToHex(C, 2); // + IntToHex(G, 4);
    self.Parent := destPanel;
    self.Visible := True;
    self.Left := X;
    self.Top := Y;
    self.Tag := G;

    N := MAX_PATH;
    SetLength( self.Name, N );
    QueryCardInfo( C, @self.DeviceID, nil, @N, PChar(self.Name) );
    SetLength( self.Name, N -1 );  // delete trailing null
    self.Num8255 := NumChannelsToNumGroups[ (DeviceID and bmNumChannels) shr 3 ];

    case (self.Num8255) of
      1: begin Note := '24 DIO bits (1 50-Pin connector)';  end;
      2: begin Note := '48 DIO bits (2 50-Pin connectors)'; end;
      3: begin Note := '72 DIO bits (3 50-Pin connectors)'; end;
      4: begin Note := '96 DIO bits (4 50-Pin connectors)'; end;
      5: begin Note := '120 DIO bits (5 50-Pin connectors)'; end;
    else
      begin
        Note := 'Unknown DIO configuration, Might not be for this sample';
      end
    end;// switch
    self.Note := Note;

    if self.Num8255>0 then begin
      BuildCardFrames( C, 0, Y );
    end;

  end;
end;


function TOne8255CardFrame.Show8255Frame(C, G, X, Y:integer): TOne8255Frame;
var
  Frame: TOne8255Frame;

begin
  Num8255 := G;
  Frame := FindComponent ('One8255Frame' + IntToHex(C,2) + IntToHex(G,4)) as TOne8255Frame;
  if not Assigned (Frame) then
  begin
    Frame := TOne8255Frame.Create (Form1);
    Frame.Parent := Cards8255Panel;
    Frame.Visible := True;
    Frame.Left := X;
    Frame.Top := Y;
    Frame.Tag := G;
    Frame.lGroupCaption.Caption := 'Group #' + IntToStr(G);
    Frame.Name := 'One8255Frame' + IntToHex(C,2) + IntToHex(G,4);
  end;
  Result := Frame;
end;

function TOne8255CardFrame.BuildCardFrames( C, X, Y: integer ): Boolean;
  var
    G:Integer;
    H:Integer;
begin
  try
    SetLength( Group, Num8255 );
    for G := 0 to high(Group) do
    begin
      Group[G] := Show8255Frame( C, G, X, 0);
      Group[G].An8255.CardNum := C;
      Group[G].An8255.GroupNum := G;
      X := X + Group[G].Width + MARGIN8255;// 4 is the "between connectors (8255s) margin")
    end;
    Width := X + 2*(bevelwidth+bevelwidth+borderwidth+MARGIN8255);//MARGIN8255*2;
    Height := CardInfoPanel.Height + Group[0].Height + 2*(bevelwidth+bevelwidth+borderwidth+MARGIN8255);
    ACardDescriptionLabel.Caption := Name + #13 + Note + #13 + '(CardNum is ' + IntToStr(C) + ')';
    Result := True;
  except
    Result := False;
  end;
end;

Begin
  RegisterClass(TOne8255CardFrame);
end.
